Imports Charmed.Sample.Models
Imports System.Text.RegularExpressions
Imports System.Threading.Tasks
Imports Windows.Web.Syndication

Namespace Charmed.Sample.Services
	''' <summary>
	''' Service to read RSS feeds.
	''' </summary>
	''' <remarks>
	''' The idea behind this class is mostly borrowed from Windows Store Blog Reader Sample
	''' http://msdn.microsoft.com/en-us/library/windows/apps/br211380.aspx
	''' </remarks>
	Public NotInheritable Class RssFeedService
		Implements IRssFeedService

		Private ReadOnly settings As ISettings

		Public Sub New(ByVal settings As ISettings)
			Me.settings = settings
		End Sub

		Public async Function GetFeedsAsync() As Task(Of List(Of FeedData)) Implements IRssFeedService.GetFeedsAsync
			Dim feeds As List(Of String) = Nothing
            Dim feedData() As String = Nothing
			If settings.TryGetValue(Of String())(Constants.FeedsKey, feedData) Then
				feeds = New List(Of String)(feedData)
			Else
				Throw New ArgumentException("There are no feeds")
			End If

            Dim feedsData As List(Of FeedData) = New List(Of FeedData)

            For Each feed As String In feeds
                Dim data As Charmed.Sample.Models.FeedData = Await GetFeedAsync(feed)
                If data IsNot Nothing Then
                    feedsData.Add(data)
                End If
            Next feed

			Return feedsData
		End Function

		Private async Function GetFeedAsync(ByVal feedUriString As String) As Task(Of FeedData)
            Dim client As SyndicationClient = New SyndicationClient
            Dim feedUri As Uri = Nothing
			If Not Uri.TryCreate(feedUriString, UriKind.RelativeOrAbsolute, feedUri) Then
				Return Nothing
			End If

			Try
                Dim feed As Windows.Web.Syndication.SyndicationFeed = Await client.RetrieveFeedAsync(feedUri)
                Dim feedData As FeedData = New FeedData
				If feed.Title IsNot Nothing AndAlso feed.Title.Text IsNot Nothing Then
					feedData.Title = feed.Title.Text
				End If
				If feed.Subtitle IsNot Nothing AndAlso feed.Subtitle.Text IsNot Nothing Then
					feedData.Description = feed.Subtitle.Text
				End If
				If feed.Items IsNot Nothing AndAlso feed.Items.Count > 0 Then
					feedData.PublishedDate = feed.Items(0).PublishedDate.DateTime

                    For Each item As Windows.Web.Syndication.SyndicationItem In feed.Items
                        Dim feedItem As FeedItem = New FeedItem
                        If item.Title IsNot Nothing AndAlso item.Title.Text IsNot Nothing Then
                            feedItem.Title = item.Title.Text
                        End If
                        If item.PublishedDate <> Nothing Then
                            feedItem.PublishedDate = item.PublishedDate.DateTime
                        End If
                        If item.Authors IsNot Nothing AndAlso item.Authors.Count > 0 Then
                            feedItem.PublishedDate = item.PublishedDate.DateTime
                        End If
                        If feed.SourceFormat = SyndicationFormat.Atom10 Then
                            If item.Content IsNot Nothing AndAlso item.Content.Text IsNot Nothing Then
                                feedItem.Content = item.Content.Text
                            End If
                            If item.Id IsNot Nothing Then
                                feedItem.Link = New Uri(item.Id)
                            End If
                        ElseIf feed.SourceFormat = SyndicationFormat.Rss20 Then
                            If item.Summary IsNot Nothing AndAlso item.Summary.Text IsNot Nothing Then
                                feedItem.Content = item.Summary.Text
                            End If
                            If item.Links IsNot Nothing AndAlso item.Links.Count > 0 Then
                                feedItem.Link = item.Links(0).Uri
                            End If
                        End If

                        feedItem.Id = Regex.Replace(feedItem.Link.ToString, "[^0-9a-zA-Z\.]+", String.Empty).GetHashCode

                        feedData.Items.Add(feedItem)
                    Next item
				End If

				Return feedData
			Catch ex As Exception
				Debug.WriteLine(ex.Message)
				Return Nothing
			End Try
		End Function
	End Class
End Namespace
