Imports Charmed.Sample.Models
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading.Tasks
Imports Windows.ApplicationModel.DataTransfer
Imports Windows.UI.StartScreen
Imports Windows.UI.Xaml
Imports System.Linq

Namespace Charmed.Sample.ViewModels
	Public NotInheritable Class FeedItemViewModel
		Inherits ViewModelBase(Of FeedItem)

		Private ReadOnly shareManager As IShareManager
		Private ReadOnly secondaryPinner As ISecondaryPinner
		Private ReadOnly storage As IStorage

		Public Sub New(ByVal shareManager As IShareManager, ByVal serializer As ISerializer, ByVal secondaryPinner As ISecondaryPinner, ByVal storage As IStorage)
			MyBase.New(serializer)
			Me.shareManager = shareManager
			Me.secondaryPinner = secondaryPinner
			Me.storage = storage
		End Sub

        Public Overrides Sub LoadState(ByVal navigationParameter As FeedItem, ByVal pageState As Dictionary(Of String, Object))
            Me.shareManager.Initialize()
            Me.shareManager.OnShareRequested = AddressOf ShareRequested

            Me.FeedItem = navigationParameter

            Me.IsFeedItemPinned = Me.secondaryPinner.IsPinned(FormatSecondaryTileId)
        End Sub

        Public Overloads Sub SaveState(ByVal pageState As Dictionary(Of String, Object))
            Me.shareManager.Cleanup()
        End Sub

        Public Async Function Pin(ByVal anchorElement As FrameworkElement) As Task
            ' Pin the feed item, then save it locally to make sure it is still available
            ' when they return.
            Dim tileInfo = New TileInfo(FormatSecondaryTileId(), Me.FeedItem.Title, Me.FeedItem.Title, TileOptions.ShowNameOnLogo OrElse TileOptions.ShowNameOnWideLogo, New Uri("ms-appx:///Assets/Logo.png"), New Uri("ms-appx:///Assets/WideLogo.png"), Me.FeedItem.Id.ToString)

            Me.IsFeedItemPinned = Await Me.secondaryPinner.Pin(anchorElement, Windows.UI.Popups.Placement.Above, tileInfo)

            If Me.IsFeedItemPinned Then
                Await SavePinnedFeedItem()
            End If
        End Function

        Public Async Function Unpin(ByVal anchorElement As FrameworkElement) As Task
            ' Unpin, then delete the feed item locally.
            Me.IsFeedItemPinned = Not Await Me.secondaryPinner.Unpin(anchorElement, Windows.UI.Popups.Placement.Above, Me.FormatSecondaryTileId)

            If Not Me.IsFeedItemPinned Then
                Await RemovePinnedFeedItem()
            End If
        End Function

        Private Function FormatSecondaryTileId() As String
            Return String.Format(Constants.SecondaryIdFormat, Me.FeedItem.Id)
        End Function

        Private Async Function SavePinnedFeedItem() As Task
            Dim pinnedFeedItems = Await Me.storage.LoadAsync(Of List(Of FeedItem))(Constants.PinnedFeedItemsKey)

            If pinnedFeedItems Is Nothing Then
                pinnedFeedItems = New List(Of FeedItem)
            End If

            pinnedFeedItems.Add(feedItem_Renamed)
            Await Me.storage.SaveAsync(Constants.PinnedFeedItemsKey, pinnedFeedItems)
        End Function

        Private Async Function RemovePinnedFeedItem() As Task
            Dim pinnedFeedItems = Await Me.storage.LoadAsync(Of List(Of FeedItem))(Constants.PinnedFeedItemsKey)

            If pinnedFeedItems IsNot Nothing Then
                Dim pinnedFeedItem = pinnedFeedItems.FirstOrDefault(Function(fi) fi.Id = Me.FeedItem.Id)
                If pinnedFeedItem IsNot Nothing Then
                    pinnedFeedItems.Remove(pinnedFeedItem)
                End If

                Await Me.storage.SaveAsync(Constants.PinnedFeedItemsKey, pinnedFeedItems)
            End If
        End Function

        Private Sub ShareRequested(ByVal dataPackage As DataPackage)
            ' Set as many data types as we can.
            dataPackage.Properties.Title = Me.FeedItem.Title

            ' Add a Uri
            dataPackage.SetUri(Me.FeedItem.Link)

            ' Add a text only version
            Dim text = String.Format("Check this out! {0} ({1})", Me.FeedItem.Title, Me.FeedItem.Link)
            dataPackage.SetText(text)

            ' Add an HTML version.
            Dim htmlBuilder = New StringBuilder
            htmlBuilder.AppendFormat("<p>Check this out!</p>", Me.FeedItem.Author)
            htmlBuilder.AppendFormat("<p><a href='{0}'>{1}</a></p>", Me.FeedItem.Link, Me.FeedItem.Title)
            Dim html = HtmlFormatHelper.CreateHtmlFormat(htmlBuilder.ToString)
            dataPackage.SetHtmlFormat(html)
        End Sub

        Private feedItem_Renamed As FeedItem
        Public Property FeedItem As FeedItem
            Get
                Return Me.feedItem_Renamed
            End Get
            Set(ByVal value As FeedItem)
                Me.SetProperty(Me.feedItem_Renamed, value)
            End Set
        End Property

        Private isFeedItemPinned_Renamed As Boolean
        Public Property IsFeedItemPinned As Boolean
            Get
                Return Me.isFeedItemPinned_Renamed
            End Get
            Set(ByVal value As Boolean)
                Me.SetProperty(Me.isFeedItemPinned_Renamed, value)
            End Set
        End Property
    End Class
End Namespace
