Imports Charmed.Container
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports System.Reflection
Imports System

Namespace Charmed
    Public NotInheritable Class Navigator
        Implements INavigator

        Private ReadOnly serializer As ISerializer
        Private ReadOnly container As IContainer

        Public Sub New(ByVal serializer As ISerializer, ByVal container As IContainer)
            Me.serializer = serializer
            Me.container = container
        End Sub

        Public Sub NavigateToViewModel(Of TViewModel)(Optional ByVal parameter As Object = Nothing) Implements INavigator.NavigateToViewModel
            Dim viewType As Type = ResolveViewType(Of TViewModel)()

            Dim frame As Frame = CType(Window.Current.Content, Frame)

            If parameter IsNot Nothing Then
                frame.Navigate(viewType, Me.serializer.Serialize(parameter))
            Else
                frame.Navigate(viewType)
            End If
        End Sub

        Public Sub GoBack() Implements INavigator.GoBack
            CType(Window.Current.Content, Frame).GoBack()
        End Sub

        Public ReadOnly Property CanGoBack As Boolean Implements INavigator.CanGoBack
            Get
                Return (CType(Window.Current.Content, Frame)).CanGoBack
            End Get
        End Property

        Private Shared Function ResolveViewType(Of TViewModel)() As Type
            Dim viewModelType As Type = GetType(TViewModel)

            Dim viewName As String = viewModelType.AssemblyQualifiedName.Replace(viewModelType.Name, viewModelType.Name.Replace("ViewModel", "Page"))

            Return Type.GetType(viewName.Replace("Model", String.Empty))
        End Function
    End Class
End Namespace
