Imports System.Threading.Tasks
Imports Windows.Storage
Imports System

Namespace Charmed
	Public NotInheritable Class Storage
		Implements IStorage

		Private ReadOnly serializer As ISerializer

		Public Sub New(ByVal serializer As ISerializer)
			Me.serializer = serializer
		End Sub

		Public async Function LoadAsync(Of T)(ByVal fileName As String) As Task(Of T) Implements IStorage.LoadAsync
            Dim file As Windows.Storage.StorageFile = Await ApplicationData.Current.LocalFolder.CreateFileAsync(fileName, CreationCollisionOption.OpenIfExists)

            Dim serializedData As String = Await FileIO.ReadTextAsync(file, Windows.Storage.Streams.UnicodeEncoding.Utf8)

			Dim data As T = Nothing
			If Not String.IsNullOrWhiteSpace(serializedData) Then
				data = Me.serializer.Deserialize(Of T)(serializedData)
			End If

			Return data
		End Function

		Public async Function SaveAsync(ByVal fileName As String, ByVal data As Object) As Task Implements IStorage.SaveAsync
            Dim file As Windows.Storage.StorageFile = Await ApplicationData.Current.LocalFolder.CreateFileAsync(fileName, CreationCollisionOption.OpenIfExists)

            Dim serializedData As String = Me.serializer.Serialize(data)

            Await FileIO.WriteTextAsync(file, serializedData, Windows.Storage.Streams.UnicodeEncoding.Utf8)
		End Function
	End Class
End Namespace
